@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Christian Kellner <gicmo@gnome.org>
  *          Samuel Cormier-Iijima <sciyoshi@gmail.com>
@@ -39,6 +37,8 @@ typedef struct _GUnixSocketAddress        GUnixSocketAddress;
 typedef struct _GUnixSocketAddressClass   GUnixSocketAddressClass;
 typedef struct _GUnixSocketAddressPrivate GUnixSocketAddressPrivate;
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GUnixSocketAddress, g_object_unref)
+
 struct _GUnixSocketAddress
 {
   GSocketAddress parent_instance;
